==> izon-mvc-4.5.2-3a8874.zip: src/Config.php <==
<?php

namespace IZON\MVC;

use Exception;
use IZON\DI\Container;
use IZON\DI\ContainerBuilder;
use IZON\Logs\Logger;
use IZON\MVC\Exceptions\Handlers\AccessForbiddenExceptionHandler;
use IZON\MVC\Exceptions\Handlers\InternalServerErrorExceptionHandler;
use IZON\MVC\Exceptions\Handlers\PageNotFoundExceptionHandler;
use IZON\MVC\Routers\Router;
use function IZON\DI\MVC\appSubDir;
use function IZON\String\endsWith;

/**
 * slouzi k nacteni konfigurace
 */
class Config {

    /**
     * pod jakym identifikatorem je ulozen router
     */
    const ROUTER_IDENTIFIER = 'mvc.router';

    /**
     * pod jakym identifikatorem jsou ulozeny spolecne interceptory pro vsechny route definition
     */
    const ROUTER_COMMON_INTERCEPTORS_IDENTIFIER = 'mvc.router.commonInterceptors';

    /**
     * jsak jsou definovany routery
     */
    const ROUTER_DEFINITIONS_INDENTIFIER = 'mvc.routeDefinitions';

    /**
     * if remove tail slashes from adresses automaticaly
     */
    const ROUTER_REDIRECT_RO_URL_WITHOUT_TAILING_SLASH_INDENTIFIER = 'mvc.router.redirectToURLWithoutTailingSlash';

    /**
     * pod jaky identifilatorem je ulozen locale resolver
     */
    const LOCALE_RESOLVER_IDENTIFIER = 'locale.resolver';
    /**
     * pod jaky identifilatorem je ulozen locale resolver
     */
    const DOMAIN_RESOLVER_IDENTIFIER = 'domain.resolver';

    /**
     * provede inicializaci loggeru
     */
    const INIT_LOGGER_IDENTIFIER = 'logs.initLogger';

    /**
     * pod jakym id jsou view resolvery
     */
    const VIEW_RESOLVER_IDENTIFIER = 'mvc.view.viewResolvers';

     /**
     * jsak jsou definovany exception handlery
     */
    const EXCEPTION_HANDLERS_INDENTIFIER = 'mvc.exceptionHandlers';

    /**
     * pod jakym idrentifikatorem je ulozan koremovy adresar aplikace
     */
    const APP_DIR_IDENTIFIER = 'app.baseDir';
    /** @deprecated use APP_DIR_IDENTIFIER instead */
    const APP_DIR = 'app.baseDir';

    /**
     * if to compile di
     */
    const COMPILE_CONFIG_INDENTIFIER = 'config.compileConfig';

    /**
     * where to store compiled di container
     */
    const COMPILED_CONFIG_CACHE_DIR_INDENTIFIER = 'di';

    /**
     * pod jakym idrentifikatorem je ulozan adresar s konfiguracenmi aplikace
     */
    const CONFIG_DIR_IDENTIFIER = 'app.configDir';

    /**
     * pod jakym idrentifikatorem je ulozan tmp adresar aplikace
     */
    const TMP_DIR_IDENTIFIER = 'app.tmpDir';

    /**
     * pod jakym idrentifikatorem je ulozan adresar cache aplikace
     */
    const CACHE_DIR_IDENTIFIER = 'app.cacheDir';

    /**
     * urcuje jestli se maji cachovat konfigy: hlavni, moduly a obsah adresate web
     */
    const USE_CONFIG_CACHE_IDENTIFIER = 'cache.useDefinitionCache';

    /**
     * do jakeho souboru se maji zapsat informce o nainstalovanych knihovnach aplikace
     */
    const MODULE_CONFIG_FILENAME = "installed-modules.php";
    /** @deprecated use MODULE_CONFIG_FILENAME */
    const INSTALLED_LIBS_FILENAME = "installed-modules.php";

    /**
     * identigikator defaultni db
     */
    const DEFAULT_DB_CONNECTION_IDENTIFIER = 'db.connection';

    /**
     * identifier of default entity manager in di
     */
    const DEFAULT_DB_ENTITY_MANAGER_IDENTIFIER = 'db.entityManager';


    /**
     * v jakem adresari se nachazi cela aplikace
     * @var string
     */
    protected $baseDir;

    /**
     * klic pod kterym je ulozeno macineid
     * @var string
     */
    protected $machineIdKey = 'MACHINE_ID';

    /**
     * v jakem akresari se nachazeji konfigy
     * @var string
     */
    protected $configDir = 'config';

    /**
     * jaky stringem zacina konfiguracni soubor
     * @var string
     */
    protected $configFilePrefix = "config";

    /**
     * v jakem adresari se nachazeji docasne soubory
     * @var string
     */
    protected $tmpDir = 'app-data/tmp';

    /**
     * @var string v jakem adresari se nachazi cache aplikace
     */
    protected $cacheDir;

    /**
     * jak se jmenuje soubor s kesi definic
     * @var string
     */
    protected $configCacheFileName = "config-cache.php";

    /**
     * container builder slouzici k vytvoreni containeru
     * @var ContainerBuilder
     */
    protected $containerBuilderClass;

    /**
     * DI container z ktereho je mozne vytahovat controllery
     * @var Container
     */
    protected $container;

    /**
     * @var array containing definitions of di container
     */
    protected $definitionsArray = [];

    /**
     *
     * @var Logger
     */
    protected $log;

    /**
     * @var boolean jestli je vyvojovy server
     */
    protected static $develserver = false;

    /**
     * @var Container|array docasna konfigurace, aby bylo mozno ji vytahnout
     */
    protected static $extConfig;


    public function __construct(
        $baseDir,
        $containerBuilderClass = ContainerBuilder::class
    ) {
        $this->baseDir = $baseDir;
        $this->containerBuilderClass = $containerBuilderClass;

        $this->cacheDir = $this->tmpDir ."/cache";
    }

    public static function loadDir($path) {
        $config = [];
        $files = glob($path.'/*.php');
        foreach($files as $file) {
            $config = array_merge($config, include_once($file));
        }
        return $config;
    }

    /**
     * initializes config
     * @throws Exception
     */
    public function init() {
        $containerBuilderClass = $this->containerBuilderClass;
        $builder = new $containerBuilderClass();

        // nastavuje zakladni cesty v aplikaci
        $pathDefinitions = [
            self::APP_DIR_IDENTIFIER => $this->baseDir,
            self::CONFIG_DIR_IDENTIFIER => appSubDir("/". $this->configDir),
            self::TMP_DIR_IDENTIFIER => appSubDir("/". $this->tmpDir),
            self::CACHE_DIR_IDENTIFIER => appSubDir("/". $this->cacheDir),
        ];
        $builder->addDefinitions($pathDefinitions);

        // nastavuje konfig
        $configDir = $this->getAbsoluteConfigDir();

        // nacte zakladni konfiguraci
        $rootConfig = $configDir ."/". $this->configFilePrefix .".php";
        if( !file_exists($rootConfig) ) {
            throw new Exception("Neexistuje základní konfig $rootConfig");
        }
        $builder->addDefinitions($rootConfig);

        // docasny konfig dokud se nevytvori di kontainer
        $tempConfig = require $rootConfig;

        // nacte specificke konfigurace podle MACHINE_ID
        if( getenv($this->machineIdKey) != '' ) {
//            $this->log->info( $this->machineIdKey ." is set to ". getenv($this->machineIdKey));

            $machineinfo = explode('-', getenv($this->machineIdKey));

            for($i = 1; $i <= count($machineinfo); $i++) {
                $tmp = '';
                for ($j = 0; $j < $i; $j++) {
                    $tmp .= '-'. $machineinfo[$j];
                }
                $filePath = $configDir . "/" . $this->configFilePrefix . $tmp . '.php';
                if( file_exists($filePath) ) {
//                    $this->log->info( "Adding config ". $filePath);

                    // nacte specifickou konfiguraci
                    $builder->addDefinitions($filePath);

                    // slit s korenovou konfiguraci
                    $tempConfig = array_merge($tempConfig, require $filePath);
                }
            }
        }

        // nastavuje develserver podle konfigu
        self::$develserver = $tempConfig["develserver"];
        self::$extConfig = $tempConfig;

        // nacist ostatni konfigurace webu
        $webDir = $this->getAbsoluteConfigDir() . "/web";
        foreach(scandir($webDir) as $file) {
            if( endsWith($file, ".php") ) {
                $builder->addDefinitions($webDir . "/" . $file);
            }
        }
        // vytvori di container
        $this->container = $builder->build();
        self::$extConfig = $this->container;

        // incializuje logger
        $this->container->get(self::INIT_LOGGER_IDENTIFIER);

        // sprovoznit logovani
        $this->log = Logger::getLogger(self::class);
    }

    /**
     *
     * @return Container
     */
    public function getContainer() {
        return $this->container;
    }

    public static function isDevelServer() {
        return self::$develserver;
    }

    /**
     * vrati hodnotu parametru z konfigu
     *
     * @param string $param Entry name
     * @return string|array
     */
    public static function getValue($param) {
        if( is_array(self::$extConfig) ) {
            return self::$extConfig[$param];
        } else {
            return self::$extConfig->get($param);
        }
    }

    /**
     * vrari router pro aplikaci
     * @return Router
     * @throws Exception
     */
    public function getRouter() {
        return $this->container->get(self::ROUTER_IDENTIFIER);

    }

    public function getLocaleResolver() {
        return $this->container->get(self::LOCALE_RESOLVER_IDENTIFIER);
    }

    public function getDomainResolver() {
        return $this->container->get(self::DOMAIN_RESOLVER_IDENTIFIER);
    }


    // TODO: cesty vyhodit do promennych tridy
    public function getViewResolvers() {
        $container = $this->container;
        if (!$container->has(self::VIEW_RESOLVER_IDENTIFIER)) {
            throw new Exception("V konfuguraci musis byt nastaveny definice routovani adres v klici 'mvc.routeDefinitions'");
        }
        return $container->get(self::VIEW_RESOLVER_IDENTIFIER);
    }

    /**
     * returns exception handlers for application
     */
    public function getExceptionHandlers() {
        $container = $this->container;
        if ( $container->has(self::EXCEPTION_HANDLERS_INDENTIFIER) ) {
            $handlers = $container->get(self::EXCEPTION_HANDLERS_INDENTIFIER);
            $this->log->info("Using config exception handlers");
        } else {
            // use default exeption handlers
            $handlers = [
                // DOTO: fix, probbably do not define defaults
//                new Exceptions\Handlers\OldPageNotFoundExceptionHandler($this->baseDir ."/www/errors/404.php"),
//                new Exceptions\Handlers\OldAccessForbiddenExceptionHandler($this->baseDir ."/www/errors/404.php"),
//                new InternalServerErrorExceptionHandler($this->baseDir ."/www/errors/500.php")
            ];

            $this->log->info("Using default exception handlers");
        }

        return $handlers;
    }


    /**
     * @return Logger returns aplication level logger
     */
    public function getAppLogger() {
        return Logger::getLogger(App::class);
    }

    /**
     * vrati basedir pro aplikaci
     * @return string
     */
    public function getBaseDir() {
        return $this->baseDir;
    }

    /**
     * vrati absolutni odkaz do nofiguracniho souboru
     * @return string
     */
    protected function getAbsoluteConfigDir() {
        return $this->baseDir ."/". $this->configDir;
    }

    /**
     * returns all keys defined in config
     * @return array
     */
    function getDefinedKeys() {
        return array_keys($this->definitionsArray);
    }

    function getConfigDir() {
        return $this->configDir;
    }

    function getCacheDir() {
        return $this->cacheDir;
    }
}

==> izon-mvc-4.5.25-9b8660.zip: src/Config.php <==
<?php

namespace IZON\MVC;

use Exception;
use IZON\DI\Container;
use IZON\DI\ContainerBuilder;
use IZON\Logs\Logger;
use IZON\MVC\Exceptions\Handlers\AccessForbiddenExceptionHandler;
use IZON\MVC\Exceptions\Handlers\InternalServerErrorExceptionHandler;
use IZON\MVC\Exceptions\Handlers\PageNotFoundExceptionHandler;
use IZON\MVC\Routers\Router;
use function IZON\DI\MVC\appSubDir;
use function IZON\String\endsWith;

/**
 * slouzi k nacteni konfigurace
 */
class Config {

    /**
     * pod jakym identifikatorem je ulozen router
     */
    const ROUTER_IDENTIFIER = 'mvc.router';

    /**
     * pod jakym identifikatorem jsou ulozeny spolecne interceptory pro vsechny route definition
     */
    const ROUTER_COMMON_INTERCEPTORS_IDENTIFIER = 'mvc.router.commonInterceptors';

    /**
     * jsak jsou definovany routery
     */
    const ROUTER_DEFINITIONS_INDENTIFIER = 'mvc.routeDefinitions';

    /**
     * if remove tail slashes from adresses automaticaly
     */
    const ROUTER_REDIRECT_RO_URL_WITHOUT_TAILING_SLASH_INDENTIFIER = 'mvc.router.redirectToURLWithoutTailingSlash';

    /**
     * pod jaky identifilatorem je ulozen locale resolver
     */
    const LOCALE_RESOLVER_IDENTIFIER = 'locale.resolver';
    /**
     * pod jaky identifilatorem je ulozen locale resolver
     */
    const DOMAIN_RESOLVER_IDENTIFIER = 'domain.resolver';

    /**
     * provede inicializaci loggeru
     */
    const INIT_LOGGER_IDENTIFIER = 'logs.initLogger';

    /**
     * pod jakym id jsou view resolvery
     */
    const VIEW_RESOLVER_IDENTIFIER = 'mvc.view.viewResolvers';

     /**
     * jsak jsou definovany exception handlery
     */
    const EXCEPTION_HANDLERS_INDENTIFIER = 'mvc.exceptionHandlers';

    /**
     * default template factory
     */
    const TEMPLATE_FACTORY = 'mvc.templateFactory';

    /**
     * pod jakym idrentifikatorem je ulozan koremovy adresar aplikace
     */
    const APP_DIR_IDENTIFIER = 'app.baseDir';
    /** @deprecated use APP_DIR_IDENTIFIER instead */
    const APP_DIR = 'app.baseDir';

    /**
     * if to compile di
     */
    const COMPILE_CONFIG_INDENTIFIER = 'config.compileConfig';

    /**
     * where to store compiled di container
     */
    const COMPILED_CONFIG_CACHE_DIR_INDENTIFIER = 'di';

    /**
     * pod jakym idrentifikatorem je ulozan adresar s konfiguracenmi aplikace
     */
    const CONFIG_DIR_IDENTIFIER = 'app.configDir';

    /**
     * pod jakym idrentifikatorem je ulozan tmp adresar aplikace
     */
    const TMP_DIR_IDENTIFIER = 'app.tmpDir';

    /**
     * pod jakym idrentifikatorem je ulozan adresar cache aplikace
     */
    const CACHE_DIR_IDENTIFIER = 'app.cacheDir';

    /**
     * urcuje jestli se maji cachovat konfigy: hlavni, moduly a obsah adresate web
     */
    const USE_CONFIG_CACHE_IDENTIFIER = 'cache.useDefinitionCache';

    /**
     * do jakeho souboru se maji zapsat informce o nainstalovanych knihovnach aplikace
     */
    const MODULE_CONFIG_FILENAME = "installed-modules.php";
    /** @deprecated use MODULE_CONFIG_FILENAME */
    const INSTALLED_LIBS_FILENAME = "installed-modules.php";

    /**
     * identigikator defaultni db
     */
    const DEFAULT_DB_CONNECTION_IDENTIFIER = 'db.connection';

    /**
     * identifier of default entity manager in di
     */
    const DEFAULT_DB_ENTITY_MANAGER_IDENTIFIER = 'db.entityManager';


    /**
     * v jakem adresari se nachazi cela aplikace
     * @var string
     */
    protected $baseDir;

    /**
     * klic pod kterym je ulozeno macineid
     * @var string
     */
    protected $machineIdKey = 'MACHINE_ID';

    /**
     * v jakem akresari se nachazeji konfigy
     * @var string
     */
    protected $configDir = 'config';

    /**
     * jaky stringem zacina konfiguracni soubor
     * @var string
     */
    protected $configFilePrefix = "config";

    /**
     * v jakem adresari se nachazeji docasne soubory
     * @var string
     */
    protected $tmpDir = 'app-data/tmp';

    /**
     * @var string v jakem adresari se nachazi cache aplikace
     */
    protected $cacheDir;

    /**
     * jak se jmenuje soubor s kesi definic
     * @var string
     */
    protected $configCacheFileName = "app-config.cache";

    /**
     * container builder slouzici k vytvoreni containeru
     * @var ContainerBuilder
     */
    protected $containerBuilderClass;

    /**
     * DI container z ktereho je mozne vytahovat controllery
     * @var Container
     */
    protected $container;

    /**
     * @var array containing definitions of di container
     */
    protected $definitionsArray = [];

    /**
     *
     * @var Logger
     */
    protected $log;

    /**
     * @var boolean jestli je vyvojovy server
     */
    protected static $develserver = false;

    /**
     * @var Container|array docasna konfigurace, aby bylo mozno ji vytahnout
     */
    protected static $extConfig;


    public function __construct(
        $baseDir,
        $containerBuilderClass = ContainerBuilder::class
    ) {
        $this->baseDir = $baseDir;
        $this->containerBuilderClass = $containerBuilderClass;

        $this->cacheDir = $this->tmpDir ."/cache";
    }

    public static function loadDir($path) {
        $config = [];
        $files = glob($path.'/*.php');
        foreach($files as $file) {
            $config = array_merge($config, include_once($file));
        }
        return $config;
    }

    /**
     * initializes config
     * @throws Exception
     */
    public function init() {
        $containerBuilderClass = $this->containerBuilderClass;
        $builder = new $containerBuilderClass();

        // nastavuje zakladni cesty v aplikaci
        $pathDefinitions = [
            self::APP_DIR_IDENTIFIER => $this->baseDir,
            self::CONFIG_DIR_IDENTIFIER => appSubDir("/". $this->configDir),
            self::TMP_DIR_IDENTIFIER => appSubDir("/". $this->tmpDir),
            self::CACHE_DIR_IDENTIFIER => appSubDir("/". $this->cacheDir),
        ];
        $builder->addDefinitions($pathDefinitions);

        // nastavuje konfig
        $configDir = $this->getAbsoluteConfigDir();

        // nacte zakladni konfiguraci
        $rootConfig = $configDir ."/". $this->configFilePrefix .".php";
        if( !file_exists($rootConfig) ) {
            throw new Exception("Neexistuje základní konfig $rootConfig");
        }
        $builder->addDefinitions($rootConfig);

        // docasny konfig dokud se nevytvori di kontainer
        $tempConfig = require $rootConfig;

        // nacte specificke konfigurace podle MACHINE_ID
        if( getenv($this->machineIdKey) != '' ) {
//            $this->log->info( $this->machineIdKey ." is set to ". getenv($this->machineIdKey));

            $machineinfo = explode('-', getenv($this->machineIdKey));

            for($i = 1; $i <= count($machineinfo); $i++) {
                $tmp = '';
                for ($j = 0; $j < $i; $j++) {
                    $tmp .= '-'. $machineinfo[$j];
                }
                $filePath = $configDir . "/" . $this->configFilePrefix . $tmp . '.php';
                if( file_exists($filePath) ) {
//                    $this->log->info( "Adding config ". $filePath);

                    // nacte specifickou konfiguraci
                    $builder->addDefinitions($filePath);

                    // slit s korenovou konfiguraci
                    $tempConfig = array_merge($tempConfig, require $filePath);
                }
            }
        }

        // nastavuje develserver podle konfigu
        self::$develserver = $tempConfig["develserver"];
        self::$extConfig = $tempConfig;

        // nacist ostatni konfigurace webu
        $webDir = $this->getAbsoluteConfigDir() . "/web";
        foreach(scandir($webDir) as $file) {
            if( endsWith($file, ".php") ) {
                $builder->addDefinitions($webDir . "/" . $file);
            }
        }
        // vytvori di container
        $this->container = $builder->build();
        self::$extConfig = $this->container;

        // incializuje logger
        $this->container->get(self::INIT_LOGGER_IDENTIFIER);

        // sprovoznit logovani
        $this->log = Logger::getLogger(self::class);
    }

    /**
     *
     * @return Container
     */
    public function getContainer() {
        return $this->container;
    }

    public static function isDevelServer() {
        return self::$develserver;
    }

    /**
     * vrati hodnotu parametru z konfigu
     *
     * @param string $param Entry name
     * @return string|array
     */
    public static function getValue($param) {
        if( is_array(self::$extConfig) ) {
            return self::$extConfig[$param];
        } else {
            return self::$extConfig->get($param);
        }
    }

    /**
     * vrari router pro aplikaci
     * @return Router
     * @throws Exception
     */
    public function getRouter() {
        return $this->container->get(self::ROUTER_IDENTIFIER);

    }

    public function getLocaleResolver() {
        return $this->container->get(self::LOCALE_RESOLVER_IDENTIFIER);
    }

    public function getDomainResolver() {
        return $this->container->get(self::DOMAIN_RESOLVER_IDENTIFIER);
    }


    // TODO: cesty vyhodit do promennych tridy
    public function getViewResolvers() {
        $container = $this->container;
        if (!$container->has(self::VIEW_RESOLVER_IDENTIFIER)) {
            throw new Exception("V konfuguraci musis byt nastaveny definice routovani adres v klici 'mvc.routeDefinitions'");
        }
        return $container->get(self::VIEW_RESOLVER_IDENTIFIER);
    }

    /**
     * returns exception handlers for application
     */
    public function getExceptionHandlers() {
        $container = $this->container;
        if( $container === null ) { // container not initialized properly
            return []; // no handlers provided
        }

        if( $container->has(self::EXCEPTION_HANDLERS_INDENTIFIER) ) {
            $handlers = $container->get(self::EXCEPTION_HANDLERS_INDENTIFIER);
            $this->log->info("Using config exception handlers");
        } else {
            // use default exeption handlers
            $handlers = [
                // DOTO: fix, probbably do not define defaults
//                new Exceptions\Handlers\OldPageNotFoundExceptionHandler($this->baseDir ."/www/errors/404.php"),
//                new Exceptions\Handlers\OldAccessForbiddenExceptionHandler($this->baseDir ."/www/errors/404.php"),
//                new InternalServerErrorExceptionHandler($this->baseDir ."/www/errors/500.php")
            ];

            $this->log->info("Using default exception handlers");
        }

        return $handlers;
    }

    public function getTemplateFactory() {
        if($this->container->has(self::TEMPLATE_FACTORY)) {
            return $this->container->get(self::TEMPLATE_FACTORY);
        }
        return null;
    }


    /**
     * @return Logger returns aplication level logger
     */
    public function getAppLogger() {
        return Logger::getLogger(App::class);
    }

    /**
     * vrati basedir pro aplikaci
     * @return string
     */
    public function getBaseDir() {
        return $this->baseDir;
    }

    /**
     * vrati absolutni odkaz do nofiguracniho souboru
     * @return string
     */
    protected function getAbsoluteConfigDir() {
        return $this->baseDir ."/". $this->configDir;
    }

    /**
     * returns all keys defined in config
     * @return array
     */
    function getDefinedKeys() {
        return array_keys($this->definitionsArray);
    }

    function getConfigDir() {
        return $this->configDir;
    }

    function getCacheDir() {
        return $this->cacheDir;
    }
}
